# names.rb
#
module FTLTools

  # Provides First and Last name based on gender and optionally culture.
  class Name

    CULTURES = %w[ humaniti ]
    attr_reader :name

    def initialize(options)
      @gender   = options['gender']
      @culture  = get_culture(options)
      @tdm      = DataMine.adapter_for(:text)
      @name     = new_name
    end

    alias to_s name

    def get_culture(options)
      if options.key?('culture') && CULTURES.include?(options['culture'])
        culture = options['culture']
      else
        culture = CULTURES.sample
      end
      culture
    end

    # Gender required but defaults to male.
    def first_name
      gender = if @gender == 'F'
                 'female'
               else
                 'male'
               end
      namefile = FTLTools.root('data/names', "#{@culture}_#{gender}_firstnames")
      @tdm.start(namefile)
      @tdm.item
    end

    def last_name
      namefile = FTLTools.root('data/names', "#{@culture}_lastnames")
      @tdm.start(namefile)
      @tdm.item
    end

    # Needs gender, produces first and last name as a single string.
    def new_name
      f_name    = first_name
      l_name    = last_name
      "#{f_name} #{l_name}"
    end

  end
end
