# ship_builder.rb

module FTLTools

  class ShipBuilder
  
    attr_reader :ship

    def initialize(data = {})
      @ship   = FTLTools::Ship.new
      @dice   = FTLTools::Dice.new
    end

    def reset
      @ship   = FTLTools::Ship.new
    end

    def setup(data = {})
      @data                 = data
      @ship.name            = @data.fetch(:name, generate_name) 
      @ship.hull_size       = @data.fetch(:hull_size, generate_hull_size).to_i
      @ship.drive_size      = @data.fetch(:drive_size, generate_drive_size).to_i
      @ship.weapons         = @data.fetch(:weapons, generate_weapons).to_i
      @ship.passengers      = @data.fetch(:passengers, generate_passengers)
      @ship.cargo_capacity  = @data.fetch(:cargo_capacity, generate_cargo_capacity).to_i
      @ship.service         = @data.fetch(:service, generate_service)
      @ship
    end

    def generate_hull_size
      @dice.roll_2 * 100
    end

    def generate_drive_size
      ((@ship.hull_size / 10) * 1.5).to_i
    end

    def generate_weapons
      @ship.hull_size / 100
    end

    def generate_name
      %w(BMM-12345 Avenger Fred).sample
    end
     
    def generate_passengers
      # Need to figure out the ship build specs, vs hull size.
      @dice.roll_2
    end 

    def generate_cargo_capacity
      hold = @dice.roll_1 * 15
      hold_max = @ship.hull_size / 10
      [hold, hold_max].min
    end

    def generate_service
      ['Free Trader', 'Navy', 'Merchant Marine'].sample
    end

  end
end
